@@ -1,8 +1,9 @@
 from decimal import Decimal, InvalidOperation
 from datetime import datetime, date, time
+from uuid import uuid4, UUID
 import re
 
-from pony.utils import is_utf8, simple_decorator, throw, localbase
+from pony.utils import is_utf8, decorator, throw, localbase
 from pony.converting import str2date, str2datetime
 from pony.orm.ormtypes import LongStr, LongUnicode
 
@@ -12,10 +13,6 @@ class DBException(Exception):
         Exception.__init__(exc, *args)
         exc.original_exc = original_exc
 
-class RowNotFound(DBException): pass
-class MultipleRowsFound(DBException): pass
-class TooManyRowsFound(DBException): pass
-
 ##StandardError
 ##        |__Warning
 ##        |__Error
@@ -39,7 +36,7 @@ class     InternalError(DatabaseError): pass
 class     ProgrammingError(DatabaseError): pass
 class     NotSupportedError(DatabaseError): pass
 
-@simple_decorator
+@decorator
 def wrap_dbapi_exceptions(func, provider, *args, **kwargs):
     dbapi_module = provider.dbapi_module
     try: return func(provider, *args, **kwargs)
@@ -57,6 +54,11 @@ def wrap_dbapi_exceptions(func, provider, *args, **kwargs):
     except dbapi_module.Error, e: raise Error(e)
     except dbapi_module.Warning, e: raise Warning(e)
 
+def unexpected_args(attr, args):
+    throw(TypeError,
+        'Unexpected positional argument%s for attribute %s: %r'
+        % ((args > 1 and 's' or ''), attr, ', '.join(map(repr, args))))
+
 version_re = re.compile('[0-9\.]+')
 
 def get_version_tuple(s):
@@ -69,10 +71,14 @@ class DBAPIProvider(object):
     paramstyle = 'qmark'
     quote_char = '"'
     max_params_count = 200
+    max_name_len = 128
 
     table_if_not_exists_syntax = True
+    index_if_not_exists_syntax = True
     max_time_precision = default_time_precision = 6
+    select_for_update_nowait_syntax = True
 
+    dialect = None
     dbapi_module = None
     dbschema_cls = None
     translator_cls = None
@@ -115,6 +121,16 @@ class DBAPIProvider(object):
             prefix = entity.__name__.lower() + '_'
             return [ prefix + column for column in columns ]
 
+    def get_default_index_name(provider, table_name, column_names, is_pk=False, is_unique=False, m2m=False):
+        if is_pk: index_name = 'pk_%s' % table_name
+        else:
+            if is_unique: template = 'unq_%(tname)s__%(cnames)s'
+            elif m2m: template = 'idx_%(tname)s'
+            else: template = 'idx_%(tname)s__%(cnames)s'
+            index_name = template % dict(tname=table_name,
+                                         cnames='_'.join(name for name in column_names))
+        return index_name[:provider.max_name_len].lower()
+
     def quote_name(provider, name):
         quote_char = provider.quote_char
         if isinstance(name, basestring):
@@ -126,6 +142,9 @@ class DBAPIProvider(object):
         builder = provider.sqlbuilder_cls(provider, ast)
         return builder.sql, builder.adapter
 
+    def should_reconnect(provider, exc):
+        return False
+
     @wrap_dbapi_exceptions
     def connect(provider):
         return provider.pool.connect()
@@ -139,18 +158,14 @@ class DBAPIProvider(object):
         return provider.pool.drop(connection)
 
     @wrap_dbapi_exceptions
-    def execute(provider, cursor, sql, arguments=None):
-        if arguments is None: cursor.execute(sql)
-        else: cursor.execute(sql, arguments)
-
-    @wrap_dbapi_exceptions
-    def executemany(provider, cursor, sql, arguments_list):
-        cursor.executemany(sql, arguments_list)
-
-    @wrap_dbapi_exceptions
-    def execute_returning_id(provider, cursor, sql, arguments):
-        cursor.execute(sql, arguments)
-        return cursor.lastrowid
+    def execute(provider, cursor, sql, arguments=None, returning_id=False):
+        if type(arguments) is list:
+            assert arguments and not returning_id
+            cursor.executemany(sql, arguments)
+        else:
+            if arguments is None: cursor.execute(sql)
+            else: cursor.execute(sql, arguments)
+            if returning_id: return cursor.lastrowid
 
     @wrap_dbapi_exceptions
     def commit(provider, connection):
@@ -180,10 +195,7 @@ class DBAPIProvider(object):
     def get_pool(provider, *args, **kwargs):
         return Pool(provider.dbapi_module, *args, **kwargs)
 
-    def set_optimistic_mode(provider, connection):
-        pass
-
-    def set_pessimistic_mode(provider, connection):
+    def set_transaction_mode(provider, connection, optimistic):
         pass
 
     def start_optimistic_save(provider, connection):
@@ -204,7 +216,7 @@ class Pool(localbase):
         assert con is pool.con
         try: con.rollback()
         except:
-            pool.close(con)
+            pool.drop(con)
             raise
     def drop(pool, con):
         assert con is pool.con
@@ -220,7 +232,7 @@ class Converter(object):
         if attr is None: return
         kwargs = attr.kwargs.copy()
         converter.init(kwargs)
-        for option in kwargs: throw(TypeError, 'Unknown option %r' % option)
+        for option in kwargs: throw(TypeError, 'Attribute %s has unknown option %r' % (attr, option))
     def init(converter, kwargs):
         attr = converter.attr
         if attr and attr.args: unexpected_args(attr, attr.args)
@@ -301,10 +313,10 @@ class IntConverter(Converter):
         Converter.init(converter, kwargs)
         min_val = kwargs.pop('min', None)
         if min_val is not None and not isinstance(min_val, (int, long)):
-            throw(TypeError, "'min' argument for attribute %s must be int. Got: %r" % (attr, min_val))
+            throw(TypeError, "'min' argument for attribute %s must be int. Got: %r" % (converter.attr, min_val))
         max_val = kwargs.pop('max', None)
         if max_val is not None and not isinstance(max_val, (int, long)):
-            throw(TypeError, "'max' argument for attribute %s must be int. Got: %r" % (attr, max_val))
+            throw(TypeError, "'max' argument for attribute %s must be int. Got: %r" % (converter.attr, max_val))
         converter.min_val = min_val
         converter.max_val = max_val
     def validate(converter, val):
@@ -335,12 +347,12 @@ class RealConverter(Converter):
         if min_val is not None:
             try: min_val = float(min_val)
             except ValueError:
-                throw(TypeError, "Invalid value for 'min' argument for attribute %s: %r" % (attr, min_val))
+                throw(TypeError, "Invalid value for 'min' argument for attribute %s: %r" % (converter.attr, min_val))
         max_val = kwargs.pop('max', None)
         if max_val is not None:
             try: max_val = float(max_val)
             except ValueError:
-                throw(TypeError, "Invalid value for 'max' argument for attribute %s: %r" % (attr, max_val))
+                throw(TypeError, "Invalid value for 'max' argument for attribute %s: %r" % (converter.attr, max_val))
         converter.min_val = min_val
         converter.max_val = max_val
         converter.tolerance = kwargs.pop('tolerance', converter.default_tolerance)
@@ -497,3 +509,26 @@ class DatetimeConverter(Converter):
         if not attr or precision == attr.entity._database_.provider.default_time_precision:
             return converter.sql_type_name
         return converter.sql_type_name + '(%d)' % precision
+
+class UuidConverter(Converter):
+    def __init__(converter, py_type, attr=None):
+        if attr is not None and attr.auto:
+            attr.auto = False
+            if not attr.default: attr.default = uuid4
+        Converter.__init__(converter, py_type, attr)
+    def validate(converter, val):
+        if isinstance(val, UUID): return val
+        if isinstance(val, buffer): return UUID(bytes=val)
+        if isinstance(val, basestring):
+            if len(val) == 16: return UUID(bytes=val)
+            return UUID(hex=val)
+        if isinstance(val, int): return UUID(int=val)
+        if converter.attr is not None:
+            throw(ValueError, 'Value type of attribute %s must be UUID. Got: %r'
+                               % (converter.attr, type(val)))
+        else: throw(ValueError, 'Expected UUID value, got: %r' % type(val))
+    def py2sql(converter, val):
+        return buffer(val.bytes)
+    sql2py = validate
+    def sql_type(converter):
+        return "UUID"
