@@ -1,16 +1,16 @@
-import os.path, weakref
+import os.path
 import sqlite3 as sqlite
-from thread import get_ident
 from threading import Lock, Thread
 from Queue import Queue
 from decimal import Decimal
-from datetime import datetime, date, time
+from datetime import datetime, date
 from time import strptime
+from uuid import UUID
 
 from pony.orm import dbschema, sqltranslation, dbapiprovider
 from pony.orm.sqlbuilding import SQLBuilder, join
-from pony.orm.dbapiprovider import DBAPIProvider, Pool, wrap_dbapi_exceptions
-from pony.utils import localbase, datetime2timestamp, timestamp2datetime, simple_decorator, absolutize_path, throw
+from pony.orm.dbapiprovider import DBAPIProvider, Pool
+from pony.utils import localbase, datetime2timestamp, timestamp2datetime, decorator, absolutize_path, throw
 
 class SQLiteForeignKey(dbschema.ForeignKey):
     def get_create_command(foreign_key):
@@ -27,6 +27,9 @@ class SQLiteTranslator(sqltranslation.SQLTranslator):
 
 class SQLiteBuilder(SQLBuilder):
     dialect = 'SQLite'
+    def SELECT_FOR_UPDATE(builder, nowait, *sections):
+        assert not builder.indent and not nowait
+        return builder.SELECT(*sections)
     def TODAY(builder):
         return "date('now', 'localtime')"
     def NOW(builder):
@@ -90,6 +93,9 @@ class SQLiteDatetimeConverter(dbapiprovider.DatetimeConverter):
 
 class SQLiteProvider(DBAPIProvider):
     dialect = 'SQLite'
+    max_name_len = 1024
+    select_for_update_nowait_syntax = False
+
     dbapi_module = sqlite
     dbschema_cls = SQLiteSchema
     translator_cls = SQLiteTranslator
@@ -106,7 +112,8 @@ class SQLiteProvider(DBAPIProvider):
         (Decimal, SQLiteDecimalConverter),
         (buffer, dbapiprovider.BlobConverter),
         (datetime, SQLiteDatetimeConverter),
-        (date, SQLiteDateConverter)
+        (date, SQLiteDateConverter),
+        (UUID, dbapiprovider.UuidConverter),
     ]
 
     def get_pool(provider, filename, create_db=False):
@@ -117,13 +124,14 @@ class SQLiteProvider(DBAPIProvider):
             # Database instance is created
 
             # the list of frames:
-            # 4 - user code: db = Database(...)
-            # 3 - cut_exception decorator
+            # 5 - user code: db = Database(...)
+            # 4 - cut_traceback decorator wrapper
+            # 3 - cut_traceback decorator
             # 2 - pony.orm.Database.__init__()
             # 1 - pony.dbapiprovider.DBAPIProvider.__init__()
             # 0 - pony.dbproviders.sqlite.get_pool()
 
-            filename = absolutize_path(filename, frame_depth=4)
+            filename = absolutize_path(filename, frame_depth=5)
             return SQLitePool(filename, create_db)
 
 provider_cls = SQLiteProvider
@@ -172,11 +180,6 @@ def _init_connection(con):
     con.text_factory = _text_factory
     con.create_function('power', 2, pow)
 
-def unexpected_args(attr, args):
-    throw(TypeError,
-        'Unexpected positional argument%s for attribute %s: %r'
-        % ((args > 1 and 's' or ''), attr, ', '.join(map(repr, args))))
-
 mem_queue = Queue()
 
 class Local(localbase):
@@ -186,7 +189,7 @@ class Local(localbase):
 
 local = Local()
 
-@simple_decorator
+@decorator
 def in_dedicated_thread(func, *args, **kwargs):
     result_holder = []
     mem_queue.put((local.lock, func, args, kwargs, result_holder))
@@ -266,9 +269,7 @@ class SqliteMemoryDbThread(Thread):
             if x is None: break
             lock, func, args, kwargs, result_holder = x
             try: result = func(*args, **kwargs)
-            except Exception, e:
-                result_holder.append(e)
-                del e
+            except Exception, e: result_holder.append(e)
             else: result_holder.append(result)
             if lock is not None: lock.release()
 
